using System;
using System.IO;
using System.Collections;
using GDepends.Analyzer;


namespace DSWAnalyzer
{
	[Plugin("DSW", Description="Visual Studio 6 Workspace")]
	public class DSWAnalyzer : DependencyAnalyzer
	{
		/// <summary>
		/// Walk through the DSW file to pull out project and project dependencies
		/// DSW files look like this (some limes snipped for space)
		/// 
		/// ##############################################################################
		/// 
		/// Project: "ProjectName"= PathToDSP.dsp - Package Owner=<4>
		/// 
		/// {{{
		///		Begin Project Dependency
		///		Project_Dep_Name ProjectName2
		///		End Project Dependency
		/// }}}
		///
		/// ###############################################################################
		/// 
		/// Each project in the DSW is seperated by the # line, 
		/// Each project has it's dependencies listed with the project's section.
		/// 
		/// </summary>
		/// <param name="textReader">Stream from DSW file</param>
		public override void Parse(TextReader textReader)
		{
			dependencies = new Hashtable();
			
			string projectLine = textReader.ReadLine();
			while(projectLine != null)
			{
				// check each line to see if a project is named
				DependencyNode node = FindProject(projectLine);
				if(node != null)
				{
					// if a project is named we scan to the end of the 
					// project section looking for dependencies
					string dependencyLine = textReader.ReadLine();
					while(dependencyLine != null &&
						  dependencyLine.IndexOf(END_PROJECT_DELIMITER) == -1)
					{
						DependencyNode childNode = FindDependency(dependencyLine);
						if(childNode != null)
						{
							// when a dependency is found we add the reference to the list
							node.Dependencies.Add(childNode);
						}
						dependencyLine = textReader.ReadLine();
					}
				}
				projectLine = textReader.ReadLine();
			}
		}

		/// <summary>
		/// Scan a line of text looking for a project entry
		/// If a project is found a DependencyNode is added to the 
		/// dependency collection if not present
		/// </summary>
		/// <param name="line">The line of text</param>
		/// <returns>The DependencyNode found on this line, null if none</returns>
		private DependencyNode FindProject(string line)
		{
			DependencyNode node = null;

			int startIndex = line.IndexOf(PROJECT_DELIMITER);

			if(startIndex >= 0)
			{
				startIndex += PROJECT_DELIMITER.Length + 1;
				int endIndex = line.IndexOf(NAME_DELIMITER, startIndex);
				
				string projectName = line.Substring(startIndex, endIndex-startIndex);

				// remove any uneeded leading and trailing characters
				projectName = projectName.Trim(DQUOTE_TRIM);
				
				node = GetOrCreateDependencyNode(projectName);
			}

			return node;
		}

		/// <summary>
		/// Scan a line of text looking for a dependency listing.
		/// If a dependency is found a DependencyNode is added to the 
		/// dependency collection if not present
		/// </summary>
		/// <param name="line">The line of text</param>
		/// <returns>The DependencyNode found on this line, null if none</returns>
		private DependencyNode FindDependency(string line)
		{
			DependencyNode node = null;

			int startIndex = line.IndexOf(DEPENDENCY_DELIMITER);
			if(startIndex > -1)
			{
				startIndex += DEPENDENCY_DELIMITER.Length + 1;
				string name = line.Substring(startIndex);
							
				node = GetOrCreateDependencyNode(name);
			}
			return node;
		}

		/// <summary>
		/// For the given name, a DependencyNode is either retrieved from the 
		/// dependency collection or created and added to the collection.
		/// </summary>
		/// <param name="name"></param>
		/// <returns></returns>
		private DependencyNode GetOrCreateDependencyNode(string name)
		{
			DependencyNode node = null;

			if(dependencies.ContainsKey(name))
			{
				node = (DependencyNode)dependencies[name];
			}
			else
			{
				node = new DependencyNode(name);
				dependencies.Add(name, node);
			}

			return node;
		}

		private const string PROJECT_DELIMITER = "Project:";
		private const string NAME_DELIMITER = "=";
		private const string END_PROJECT_DELIMITER = "#######";
		private const string DEPENDENCY_DELIMITER = "Project_Dep_Name";
		private static readonly char[] DQUOTE_TRIM = {'\"'};


	}
}
