using System;
using System.IO;
using System.Collections;
using System.Reflection;

namespace GDepends.Analyzer
{

    /// <summary>
    /// DependencyAnalyzer is an abstract base class providing the 
    /// interface for derived classes who parse and display dependency
    /// information.
    /// </summary>
    public abstract class DependencyAnalyzer
    {   
        /// <summary>
        /// Override Parse in a derived class
        /// </summary>
        public abstract void Parse(TextReader textReader);

        /// <summary>
        ///  Graph uses Visio automation to display the dependencies.
        /// </summary>
        public virtual void Display()
        {
            LoadVisio();
            
            // create a shape for each entry in the collection
            foreach(DictionaryEntry d in dependencies)
            {
                DependencyNode depNode = (DependencyNode)d.Value;
                if(depNode.Visible)
                {
                    DropShape(depNode);
                }
            }

            // reiterate the collection, this time adding connectors 
            // from the parent to the child
            foreach(DictionaryEntry d in dependencies)
            {
                DependencyNode parentNode = (DependencyNode)d.Value;
                
                if(parentNode.Visible)
                {
                    foreach(DependencyNode childNode 
                                in parentNode.Dependencies)
                    {
                        if(childNode.Visible)
                        {
                            Visio.IVShape connector = DropConnector();
                            GlueShapes(parentNode.Shape, 
                                       childNode.Shape, 
                                       connector);
                        }
                    }
                }
            }

            LayoutPage();
        }

        /// <summary>
        /// Layout page is invoked after all shaped have been added to 
        /// Visio. Override this function to perform different formatting 
        /// and layout.
        /// </summary>
        protected virtual void LayoutPage()
        {
            // Set the page to layout as "Circular"
            // In Visio: (Shape menu -> Layout Shapes -> Style).
            // Flowchart/Tree works well also
            Visio.IVShape sheet = visioPage.PageSheet;
            sheet.get_Cells(CELL_ROUTESTYLE).Formula = ROUTESTYLE_CIRCULAR;

            visioPage.Layout();
        }

        /// <summary>
        /// GlueShapes is invoked to "attach" a parent and child shape.
        /// Override this function to glue the connector to a different point
        /// </summary>
        /// <param name="parent">Parent shape</param>
        /// <param name="child">Child shape</param>
        /// <param name="connector">Connector shape</param>
        protected virtual void GlueShapes(Visio.IVShape parent, 
                                          Visio.IVShape child, 
                                          Visio.IVShape connector)
        {
            Visio.IVCell beginCell = connector.get_Cells(CELL_BEGINX);
            Visio.IVCell endCell = connector.get_Cells(CELL_ENDX);

            Visio.IVCell topCell = child.get_Cells(CELL_PINX);
            Visio.IVCell bottomCell = parent.get_Cells(CELL_PINX);

            beginCell.GlueTo(bottomCell);
            endCell.GlueTo(topCell);
        }

        /// <summary>
        /// DropConnector is invoked to create a connector in visio
        /// Override this function to drop a different shape, 
        /// or use a different format
        /// </summary>
        /// <returns>The newly created connector shape</returns>
        protected virtual Visio.IVShape DropConnector()
        {   
            Visio.IVShape shape = visioPage.Drop(visioConnector, 4, 4);
            shape.get_Cells(CELL_ENDARROW).Formula = ENDARROW_THICK;
            return shape;
        }

        /// <summary>
        /// DropShape is invoked to create a box in Visio
        /// Override this function to drop a different shape, 
        /// or use a different format
        /// </summary>
        /// <param name="node">The DependencyNode to tie to shape</param>
        /// <returns>The newly created shape</returns>
        protected virtual Visio.IVShape DropShape(DependencyNode node)
        {
            Visio.IVShape shape = visioPage.Drop(visioBox, 4, 4);
            shape.Text = node.Name;
            node.Shape = shape;
            return shape;
        }

        /// <summary>
        /// LoadVisio starts the Visio applcation, adds a new blank 
        /// page, and loads the default masters used by this class 
        /// to draw boxes and connectors
        /// </summary>
        protected virtual void LoadVisio()
        {
            visioApp = new Visio.__Application();
            visioApp.Documents.Add("");
            visioPage = visioApp.ActivePage;
            
            Visio.IVDocument visioDoc;
            visioDoc = visioApp.Documents.Add(BLOCK_STENCIL);

            visioBox = visioDoc.Masters[AUTOSIZE_BOX];
            visioConnector = visioDoc.Masters[DYNAMIC_CONNECTOR];
        }

        /// <summary>
        /// Hashtable of DependencyNodes
        /// </summary>
        public Hashtable Dependencies
        {
            get
            {
                return dependencies;
            }
        }

        protected Hashtable dependencies;
        protected Visio.IVApplication visioApp;
        protected Visio.IVPage visioPage;
        protected Visio.IVMaster visioBox;
        protected Visio.IVMaster visioConnector;
        
        #region Visio String Constants
        const string BLOCK_STENCIL = "Blocks.vss";
        const string AUTOSIZE_BOX = "Auto-size box";
        const string DYNAMIC_CONNECTOR = "Dynamic connector";
        const string CELL_BEGINX = "BeginX";
        const string CELL_ENDX = "EndX";
        const string CELL_PINY = "PinY";
        const string CELL_PINX = "PinX";
        const string CELL_ENDARROW = "EndArrow";
        const string ENDARROW_THICK = "4";
        const string CELL_ROUTESTYLE = "RouteStyle";
        const string ROUTESTYLE_CIRCULAR = "16";
        #endregion
    }
	
    /// <summary>
    /// A DependencyNode keeps state for a dependency: name, 
    /// visibility, visio shape, and child dependencies
    /// </summary>
    public class DependencyNode
    {
        public DependencyNode(string name)
        {
            if(name == null)
            {
                throw new ArgumentNullException();
            }
            this.name = name;
            visible = true;
            dependencies = new ArrayList();
        }

        public string Name
        {
            get { return name;  }
            set { name = value; }
        }

        public bool Visible
        {
            get { return visible; }
            set { visible = value; }
        }

        public Visio.IVShape Shape
        {
            get { return shape; }
            set { shape = value; }
        }

        public ArrayList Dependencies
        {
            get { return dependencies; }
        }

        protected string name;
        protected bool visible;
        protected Visio.IVShape shape;
        protected ArrayList dependencies;
    };

	/// <summary>
	/// AnalyzerFactory tracks the analyzer plugins as they
	/// are loaded, and serves as a class factory for 
	/// DependencyAnalyzer derived objects
	/// </summary>
    public class AnalyzerFactory
    {
        /// <summary>
        ///  Searches the given assembly parameter for any custom 
        ///  PluginAttributes. Any PluginAttributes it finds are added 
        ///  to the attributeList collection.
        /// </summary>
        /// <param name="assembly">The assembly to search</param>
        /// <param name="attributeList">Plugins are added to this 
        ///                             collection</param>
        public static void RegisterAssembly(Assembly assembly, 
                                            ArrayList attributeList)
        {           
            Type [] types = assembly.GetExportedTypes();

            foreach(Type type in types)
            {
                object [] attributes = type.GetCustomAttributes(false);

                foreach(object attribute in attributes)
                {
                    if(attribute is PluginAttribute)
                    {
                        PluginAttribute plugin;
                        plugin = (PluginAttribute)attribute;
                        analyzers.Add(plugin.FileExtension.ToUpper(), 
                                      type);
                        attributeList.Add(plugin);
                    }
                }
            }
        }

        /// <summary>
        /// Returns the registered analyzer for the given file type
        /// </summary>
        /// <param name="fileType">The file type</param>
        /// <returns>A class derived from DependencyAnalyzer</returns>
        public static DependencyAnalyzer CreateAnalyzer(
                                                string fileExtension)
        {
            Type type = (Type)analyzers[fileExtension.ToUpper()];
            return (DependencyAnalyzer)Activator.CreateInstance(type);
        }

        private static Hashtable analyzers = new Hashtable();
    }

    /// <summary>
    /// This custom attribute is applied to classes implementing 
    /// DependencyAnalyzer. It allows the class to announce 
    /// the supported file extension.
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true)]
    public class PluginAttribute : System.Attribute
    {
        public PluginAttribute(string fileExtension)
        {
            this.fileExtension = fileExtension;
        }

        public string FileExtension
        {
            get { return fileExtension; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        private string fileExtension;
        private string description;
    };
}
